##############################################################################
##############################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# R program (does not use any BUGS model)
# A. salmon life cycle model
# Estimate probability of extinction for various values of harvest rate h
# WITH GENERATION OVERLAP
# Chapter 12

##############################################################################
##############################################################################


library(tcltk)

oldpath<-getwd()

source("routineSR2.R")

rythme <- read.table(file="rythme.txt")

# ----------------------------------------------------------------------------------
# Parameterization
# ----------------------------------------------------------------------------------

# Adults -> eggs

pf1 <- 0.34 	# proportion female
pf2 <- 0.68
fec1 <- 4635	# nb eggs / female
fec2 <- 7965

# Recrutement eggs --> smolts

surf <- 209400 # surface in m2
alpha1 <- -5.44 
alpha2 <- -6.71
beta <- 0.0163/surf
p <- 0.155
sigma <- 0.58

# pre-smolt ->smolt1
rho = 0.89

# smolt -> spawners
pi11<-0.13
pi12<-0.03
pi21<-0.22
pi22<-0.04

# Characteristics
# ----------------------------------------------------

# juveniles --> eggs
psi00=(rho*pi11+(1-rho)*pi21)*(pf1*fec1)+(rho*pi12+(1-rho)*pi22)*(pf2*fec2)
# equivalent ricker eggs -> juveniles
alphaprime=(sigma^2)/2+log((1-p)*exp(alpha1)+p*exp(alpha2))
betaprime=beta
# find optimal harvest rate
htest=seq(0,0.9,by=0.0001)
alphatest=htest-log(1-htest)
hopt=htest[which.min((log(psi00)+alphaprime-alphatest)^2)]
Wopt=hopt/betaprime
Nopt=SR2(Wopt,alphaprime,alphaprime,betaprime,0.5,0.00001,1)*(1-hopt)
Wopt/psi00
Nh=((log(1-hopt)+log(psi00)+alphaprime)/(betaprime*(1-hopt)*psi00))
SR2(Wopt,alphaprime,alphaprime,betaprime,0.5,0.00001,1)*psi00*(1-hopt)-Wopt


# Simulations
# -----------------------------------------------------

Nyears <- 1005
Nrep <- 1000

findeserie<-matrix(NA, ncol=6, nr=Nrep)
period=(Nyears-1001):(Nyears-2)
periodmean=(Nyears-1001+20):(Nyears-802)

h=0
hvarie=seq(0,0.40,l=21)

pourcentvarie=c(50,100,200)/100

aa=alphaprime+log(psi00)+log(1-hvarie)
bb=betaprime*psi00*(1-hvarie)
AlevinsMeanTheo=aa/bb
OeufsMeanTheo=aa/bb*psi00*(1-hvarie)

pextinct1=matrix(0,length(pourcentvarie), length(hvarie))
pextinct2=pextinct1
Alevinsstatio=pextinct1
Oeufsstatio=pextinct1
Alevinsmean=pextinct1
Alevinsvar=pextinct1
Alevinsq10=pextinct1
Oeufsmean=pextinct1
Oeufsq10=pextinct1
HM2mean=pextinct1
HM2q10=pextinct1
HM1mean=pextinct1

ptm <- proc.time()

pb <- tkProgressBar(title = "R progression", width = 420,
                    min = min(pourcentvarie), max = max(pourcentvarie), initial = min(pourcentvarie))

# loop on initial conditions

indicepourcent=0

for (pourcent in pourcentvarie) {

indicepourcent = indicepourcent+1
print(pourcentvarie[indicepourcent])
print(ptm-proc.time())
ptm<-proc.time()

initoeuf<-Wopt*pourcent
initalevins=ceiling(Nh*pourcent)
initsmolt1=ceiling(rho*Nh*pourcent)
initsmolt2=initalevins-initsmolt1
inithm11=ceiling(pi11*initsmolt1)
inithm12=ceiling(pi12*initsmolt1)
inithm21=ceiling(pi21*initsmolt2)
inithm22=ceiling(pi22*initsmolt2)

indiceh=0
for (h in hvarie) {

setTkProgressBar(pb,h,label = paste("Running for", length(hvarie)," values of h for initial conditions = ",which(pourcentvarie==pourcent)," on ", length(pourcentvarie)))

indiceh=indiceh+1
inithm1=ceiling((inithm11+inithm21))
#*(1-h))
inithm2=ceiling((inithm12+inithm22))
#*(1-h))
initoeuf<-ceiling((inithm1*fec1*pf1+inithm2*fec2*pf2)*(1-h))

Oeufs<-matrix(initoeuf,ncol=Nyears,nr=Nrep)
Alevins<-matrix(initalevins,ncol=Nyears,nr=Nrep)
Smolt1<-matrix(initsmolt1,ncol=Nyears,nr=Nrep)
Smolt2<-matrix(initsmolt2,ncol=Nyears,nr=Nrep)
SW11<-matrix(inithm11,ncol=Nyears,nr=Nrep)
SW12<-matrix(inithm12,ncol=Nyears,nr=Nrep)
SW21<-matrix(inithm21,ncol=Nyears,nr=Nrep)
SW22<-matrix(inithm22,ncol=Nyears,nr=Nrep)
HM2<-matrix(inithm2,ncol=Nyears,nr=Nrep)
HM1<-matrix(inithm1,ncol=Nyears,nr=Nrep)
#fin d Initialisation
Oeufs[,(Nyears-5):Nyears]<-findeserie
Alevins[,(Nyears-5):Nyears]<-findeserie
Smolt1[,(Nyears-5):Nyears]<-findeserie
Smolt2[,(Nyears-5):Nyears]<-findeserie
SW11[,(Nyears-5):Nyears]<-findeserie
SW12[,(Nyears-5):Nyears]<-findeserie
SW21[,(Nyears-5):Nyears]<-findeserie
SW22[,(Nyears-5):Nyears]<-findeserie
HM2[,(Nyears-5):Nyears]<-findeserie
HM1[,(Nyears-5):Nyears]<-findeserie


# States-dynamics t --> t+1 et t+2
# -----------------------------------------------------------------


for (t in 1:(Nyears-2)){
HM1[,t]<-floor((SW21[,t]+SW11[,t]))
HM2[,t]<-floor((SW22[,t]+SW12[,t]))

Oeufs[,t]<-floor((HM1[,t]*pf1*fec1+HM2[,t]*pf2*fec2)*(1-h))
#projection
#SR
Alevins[,t+1]<- SR2(Oeufs[,t],alpha1,alpha2,beta, p,sigma,Nrep)	
#Smolts
Smolt1[,t+1]<-rbinom(Nrep,Alevins[,t],rho)
Smolt2[,t+2]<-(Alevins[,t])-Smolt1[,t+1]
#Retours ulterieurs
SW12[,t+2]<-rbinom(Nrep,Smolt1[,t],pi12)
SW11[,t+1]<-rbinom(Nrep,Smolt1[,t],pi11)
SW22[,t+2]<-rbinom(Nrep,Smolt2[,t],pi22)
SW21[,t+1]<-rbinom(Nrep,Smolt2[,t],pi21)

} # end loop on t

zeros=which(Alevins[,Nyears-2]==0)
Tsurv=(Nrep-length(zeros))*length(period)-min(period-1)*length(zeros)
for (i in zeros){
Tsurv=Tsurv+min(which(Alevins[i,period]==0))

} # end loop on i

pextinct1[indicepourcent,indiceh]=length(zeros)/Nrep
pextinct2[indicepourcent,indiceh]=(Nrep*length(period)-Tsurv)/(Nrep*length(period))
Alevinsstatio[indicepourcent,indiceh]=mean(Alevins[-zeros,periodmean])
Oeufsstatio[indicepourcent,indiceh]=mean(Oeufs[-zeros,periodmean])
Alevinsmean[indicepourcent,indiceh]=mean(Alevins[,periodmean])
Alevinsvar[indicepourcent,indiceh]=var(as.vector(Alevins[,periodmean]))
Oeufsmean[indicepourcent,indiceh]=mean(Oeufs[,periodmean])
Alevinsq10[indicepourcent,indiceh]=mean(apply(Alevins[,periodmean],2,quantile,probs=c(0.1)))
Oeufsmean[indicepourcent,indiceh]=mean(Oeufs[,periodmean])
Oeufsq10[indicepourcent,indiceh]=mean(apply(Alevins[,periodmean],2,quantile,probs=c(0.1)))
HM2mean[indicepourcent,indiceh]=mean(HM2[,periodmean])
HM2q10[indicepourcent,indiceh]=mean(apply(HM2[,periodmean],2,quantile,probs=c(0.1)))
HM1mean[indicepourcent,indiceh]=mean(HM1[,periodmean])

} # end loop on h

} # end loop on pourcent



# -----------------------------------------------------
# Save result for graph

p_extinct_with_overlap <- pextinct1[2,]


