##############################################################################
##############################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# A. salmon life cycle model
# Estimate probability of extinction for various values of harvest rate h
# WITHOUT GENERATION OVERLAP
# Chapter 12
 
##############################################################################
##############################################################################


library(tcltk)

oldpath<-getwd()

source("routineSR2.R")

rythme <- read.table(file="rythme.txt")

# ----------------------------------------------------------------------------------
# Parameterization
# ----------------------------------------------------------------------------------

# Adults -> eggs

pf1 <- 0.34 	# proportion female
pf2 <- 0.68
fec1 <- 4635	# nb eggs / female
fec2 <- 7965

# Recrutement eggs --> smolts

surf <- 209400 # surface in m2
alpha1 <- -5.44 
alpha2 <- -6.71
beta <- 0.0163/surf
p <- 0.155
sigma <- 0.58

# pre-smolt ->smolt1
rho = 0.89

# smolt -> spawners
pi11<-0.13
pi12<-0.03
pi21<-0.22
pi22<-0.04

# Characteristics
# ----------------------------------------------------

# juveniles --> eggs
psi00=(rho*pi11+(1-rho)*pi21)*(pf1*fec1)+(rho*pi12+(1-rho)*pi22)*(pf2*fec2)
# equivalent ricker eggs -> juveniles
alphaprime=(sigma^2)/2+log((1-p)*exp(alpha1)+p*exp(alpha2))
betaprime=beta
# find optimal harvest rate
htest=seq(0,0.9,by=0.0001)
alphatest=htest-log(1-htest)
hopt=htest[which.min((log(psi00)+alphaprime-alphatest)^2)]
Wopt=hopt/betaprime
Nopt=SR2(Wopt,alphaprime,alphaprime,betaprime,0.5,0.00001,1)*(1-hopt)
Wopt/psi00
Nh=((log(1-hopt)+log(psi00)+alphaprime)/(betaprime*(1-hopt)*psi00))
SR2(Wopt,alphaprime,alphaprime,betaprime,0.5,0.00001,1)*psi00*(1-hopt)-Wopt


# Simulations
# -----------------------------------------------------

Nyears <- 1005
Nrep <- 1000

findeserie<-matrix(NA, ncol=6, nr=Nrep)
period=(Nyears-1001):(Nyears-2)
periodmean=(Nyears-1001):(Nyears-802)
h=0

h=0
hvarie=seq(0,0.40,l=21)

pourcentvarie=c(50,100,200)/100

aa=alphaprime+log(psi00)+log(1-hvarie)
bb=betaprime*psi00*(1-hvarie)
AlevinsMeanTheo=aa/bb
OeufsMeanTheo=aa/bb*psi00*(1-hvarie)

pextinct1=matrix(0,length(pourcentvarie), length(hvarie))
pextinct2=pextinct1
Alevinsstatio=pextinct1
Oeufsstatio=pextinct1
Alevinsmean=pextinct1
Oeufsmean=pextinct1

ptm <- proc.time()

pb <- tkProgressBar(title = "R progression", width = 400,
                    min = min(pourcentvarie), max = max(pourcentvarie), initial = min(pourcentvarie))


# loop on initial conditions

indicepourcent=0

for (pourcent in pourcentvarie) {

indicepourcent=indicepourcent+1

initoeuf<-Wopt*pourcent
initalevins<-Nh*pourcent

indiceh=0
for (h in hvarie) {

setTkProgressBar(pb,h,label = paste("Running for", length(hvarie)," values of h for initial conditions = ",which(pourcentvarie==pourcent)," on ", length(pourcentvarie)))

indiceh=indiceh+1
Oeufs<-matrix(initoeuf,ncol=Nyears,nr=Nrep)
Alevins<-matrix(initalevins,ncol=Nyears,nr=Nrep)
Oeufs[,(Nyears-5):Nyears]<-findeserie
Alevins[,(Nyears-5):Nyears]<-findeserie


# States-dynamics t --> t+1 et t+2
# -----------------------------------------------------------------

for (t in 1:(Nyears-2)){
Alevins[,t+1]<- SR2(Oeufs[,t],alpha1,alpha2,beta, p,sigma,Nrep)
Oeufs[,t+1]<-floor(Alevins[,t+1]*psi00*(1-h))

} # end loop on t


zeros=which(Alevins[,Nyears-2]==0)
Tsurv=(Nrep-length(zeros))*length(period)-min(period-1)*length(zeros)
for (i in zeros){
Tsurv=Tsurv+min(which(Alevins[i,period]==0))

} # end loop on i

pextinct1[indicepourcent,indiceh]=length(zeros)/Nrep
pextinct2[indicepourcent,indiceh]=(Nrep*length(period)-Tsurv)/(Nrep*length(period))
Alevinsstatio[indicepourcent,indiceh]=mean(Alevins[-zeros,periodmean])
Oeufsstatio[indicepourcent,indiceh]=mean(Oeufs[-zeros,periodmean])
Alevinsmean[indicepourcent,indiceh]=mean(Alevins[,periodmean])
Oeufsmean[indicepourcent,indiceh]=mean(Oeufs[,periodmean])

} # end loop on i

} # end loop on pourcent


# -----------------------------------------------------
# Save result for graph

p_extinct_without_overlap <- pextinct1[2,]

